@@ -93,6 +93,14 @@ class IJobDefinition(metaclass=ABCMeta):
         This value can be None
         """
 
+    @abstractproperty
+    def shell(self):
+        """
+        Shell that is used to interpret the command
+
+        Defaults to 'bash' for checkbox compatibility.
+        """
+
     @abstractproperty
     def command(self):
         """
@@ -356,6 +364,138 @@ class IJobRunner(metaclass=ABCMeta):
         # executing. We could expose the underlying process mechanics so that
         # QT/GTK applications could tie that directly into their event loop.
 
+    @abstractmethod
+    def get_warm_up_sequence(self, job_list):
+        """
+        Determine if authentication warm-up may be needed.
+
+        :param job_lits:
+            A list of jobs that may be executed
+        :returns:
+            A list of methods to call to complete the warm-up step.
+
+        Authentication warm-up is related to the plainbox-secure-launcher-1
+        program that can be 'warmed-up' to perhaps cache the security
+        credentials. This is usually done early in the testing process so that
+        we can prompt for passwords before doing anything that takes an
+        extended amount of time.
+        """
+
+
+class IJobRunnerUI(metaclass=ABCMeta):
+    """
+    User interface (textual) related to running a single job
+    """
+
+    @abstractmethod
+    def considering_job(self, job, job_state):
+        """
+        Method called as the runner is considering the specified job can
+        run or not.
+        """
+
+    @abstractmethod
+    def about_to_start_running(self, job, job_state):
+        """
+        Method called as the runner has decided to run the job and is
+        getting ready to start.
+        """
+
+    @abstractmethod
+    def wait_for_interaction_prompt(self, job):
+        """
+        Method called only for user-interact and user-interact-verify jobs
+        that should instruct the user to read the description (that needs
+        to be displayed somehow) and confirm before the test is actually
+        started. The user should be one of the few choices listed below:
+
+        quit:
+            save and quit
+        run:
+            run the job
+        skip:
+            skip and continue
+
+        :returns:
+            The action selected by the user.
+        """
+
+    @abstractmethod
+    def started_running(self, job, job_state):
+        """
+        Method called immediately before the runner starts to run the job
+        """
+
+    @abstractmethod
+    def about_to_execute_program(self, args, kwargs):
+        """
+        Method called just prior to execute an external program.
+
+        :param args:
+            Same as for subprocess.call
+        :param kwargs:
+            Same as for subprocess.call
+        """
+
+    @abstractmethod
+    def got_program_output(self, stream_name, line):
+        """
+        Method called on every line of output from an external program
+
+        :param stream_name:
+            either 'stdin' or 'stdout'
+        :param line:
+            the full text of the intercepted line
+        """
+
+    @abstractmethod
+    def finished_executing_program(self, returncode):
+        """
+        Method called just after running an external program
+
+        :param returncode:
+            The return code of the external program
+        """
+
+    @abstractmethod
+    def notify_about_description(self, job):
+        """
+        Method called prior to user interactions that might require familiarity
+        of the job description.
+        """
+
+    @abstractmethod
+    def finished_running(self, job, job_state, job_result):
+        """
+        Method called immediately after the runner finishes to run the job
+        """
+
+    @abstractmethod
+    def job_cannot_start(self, job, job_state, job_result):
+        """
+        Method called when a job cannot be started
+        """
+
+    @abstractmethod
+    def finished(self, job, job_state, job_result):
+        """
+        Method called at the end of the process, regardless if the job was
+        actually started or not
+        """
+
+    @abstractmethod
+    def pick_action_cmd(self, action_list, prompt=None):
+        """
+        Present a list of actions and let the user pick one
+
+        :param action_list:
+            A list of 3-tuples (accel, label, cmd)
+        :prompt:
+            An optional prompt string
+        :returns:
+            cmd of the selected action or None
+        """
+
 
 class IUserInterfaceIO(metaclass=ABCMeta):
     """
@@ -434,6 +574,14 @@ class IProviderBackend1(metaclass=ABCMeta):
         May be None
         """
 
+    @abstractproperty
+    def units_dir(self):
+        """
+        absolute path of the units directory
+
+        May be None
+        """
+
     @abstractproperty
     def secure(self):
         """
@@ -549,6 +697,16 @@ class IProvider1(metaclass=ABCMeta):
         Discover and return all executables offered by this provider
         """
 
+    def get_units(self):
+        """
+        Get all units.
+
+        :returns:
+            Pair (unit_list, problem_list) where unit_list is a unsorted list
+            of units, as they showed up in subsequent unit definition files and
+            each item from problem_list is an exception.
+        """
+
 
 class ISessionStateController(metaclass=ABCMeta):
     """
@@ -652,6 +810,22 @@ class IExecutionController(metaclass=ABCMeta):
             The higher the value, the more applicable this controller is.
         """
 
+    @abstractmethod
+    def get_warm_up_for_job(self, job):
+        """
+        Get a warm-up function that should be called before running this job.
+
+        :returns:
+            A callable (without arguments) or None, depending on needs of a
+            particular job.
+
+        The warm-up function is an optional advisory interface to improve the
+        testing experience for the user. A job may not require any warm-up. In
+        such case the return value is None. Note that even if this function is
+        not called the testing process should perform the same way (correctly)
+        but the user may be prompted for additional steps mid-way.
+        """
+
 
 class IBuildSystem(metaclass=ABCMeta):
     """
@@ -714,6 +888,63 @@ class IBuildSystem(metaclass=ABCMeta):
         """
 
 
+class ISessionStateExporter(metaclass=ABCMeta):
+    """
+    Interface for classes that export session state to a byte stream.
+
+    Exporters write out the state of the session after all jobs have finished
+    running, in a user-selected format. The intent is not to preserve
+    everything that the session may hold but instead to present it to the user
+    in the best format possible.
+
+    Each exporter can support a set of options that can alter the way it
+    operates. Options can either be set boolean-like, or they can be assigned a
+    value (a string). If an option contains a "=", the part of the string on
+    the right of the equal sign will be assigned as the option's value;
+    otherwise they operate in a boolean fashion.
+
+    It's best to keep the list of exporter options under control to keep the
+    user interface from becoming annoying.
+    """
+
+    @abstractmethod
+    # @classproperty -- this is a class-level property
+    def supported_option_list(cls):
+        """
+        Return the list of supported options
+        """
+
+    @abstractmethod
+    def get_session_data_subset(self, session):
+        """
+        Compute a subset of session data.
+
+        The subset of the data that should be saved may depend on a particular
+        saver class and options selected by the user.
+
+        Must return a collection that can be handled by :meth:`dump()`.
+        Special care must be taken when processing io_log (and in the future,
+        attachments) as those can be arbitrarily large.
+        """
+
+    @abstractmethod
+    def dump(self, data, stream):
+        """
+        Dump data to stream.
+
+        This method operates on data that was returned by
+        :meth:`get_session_data_subset()`. It may not really process bytes or
+        simple collections. Instead, for efficiency, anything is required.
+
+        As in get_session_data_subset() it's essential to safely save
+        arbitrarily large data sets (or actually, only where it matters the
+        most, like in io_log).
+
+        Data is a text stream suitable for writing.
+        """
+        # TODO: Add a way for the stream to be binary as well.
+
+
 class ISessionStateTransport(metaclass=ABCMeta):
     """
     Interface for transports that send test data somewhere.
@@ -772,3 +1003,144 @@ class ISessionStateTransport(metaclass=ABCMeta):
             gain special semantics that can be further standardized. As of this
             writing there are no standard keys.
         """
+
+
+class ITranslator(metaclass=ABCMeta):
+    """
+    Interface for all translators
+    """
+
+    @abstractmethod
+    def gettext(self, msgid):
+        """
+        Translate a message
+
+        :param msgid:
+            Identifier of the message to translate
+        :returns:
+            Translated message or msgid if translation is not available
+        """
+
+    @abstractmethod
+    def ngettext(self, msgid1, msgid2, n):
+        """
+        Translate a message involving plural form
+
+        :param msgid1:
+            Identifier of the singular form of the message to translate
+        :param msgid2:
+            Identifier of the plural form of message to translate
+        :param n:
+            Any integer number
+        :returns:
+            Translated message appropriate for the specified number, if
+            available.  If the translated number is not available one of msgid1
+            and msgid2 are returned, depending on the value of n.
+        """
+
+    # Context aware gettext + ngettext
+
+    @abstractmethod
+    def pgettext(self, msgctxt, msgid):
+        """
+        Translate a message within a context.
+
+        :param msgctxt:
+            Context that specifies which translation of msgid to pick
+        :param msgid:
+            Identifier of the message to translate
+        :returns:
+            Translated message or msgid if translation is not available
+        """
+
+    @abstractmethod
+    def pngettext(self, msgctxt, msgid1, msgid2, n):
+        """
+        Translate a message involving plural form
+
+        :param msgctxt:
+            Context that specifies which translation of msgid1/msgid2 to pick
+        :param msgid1:
+            Identifier of the singular form of the message to translate
+        :param msgid2:
+            Identifier of the plural form of message to translate
+        :param n:
+            Any integer number
+        :returns:
+            Translated message appropriate for the specified number, if
+            available. If the translated number is not available one of msgid1
+            and msgid2 are returned, depending on the value of n.
+        """
+
+    # Explicit domain gettext + ngettext
+
+    @abstractmethod
+    def dgettext(self, domain, msgid):
+        """
+        Translate a message using a specific domain
+
+        :param domain:
+            Name of the domain from which translations are obtained
+        :param msgid:
+            Identifier of the message to translate
+        :returns:
+            Translated message or msgid if translation is not available
+        """
+
+    @abstractmethod
+    def dngettext(self, domain, msgid1, msgid2, n):
+        """
+        Translate a message involving plural form using a specific domain
+
+        :param domain:
+            Name of the domain from which translations are obtained
+        :param msgid1:
+            Identifier of the singular form of the message to translate
+        :param msgid2:
+            Identifier of the plural form of message to translate
+        :param n:
+            Any integer number
+        :returns:
+            Translated message appropriate for the specified number, if
+            available.  If the translated number is not available one of msgid1
+            and msgid2 are returned, depending on the value of n.
+        """
+
+    # Explicit domain and context gettext + ngettext
+
+    @abstractmethod
+    def pdgettext(self, msgctxt, domain, msgid):
+        """
+        Translate a message using a specific context and domain
+
+        :param msgctxt:
+            Context that specifies which translation of msgid to pick
+        :param domain:
+            Name of the domain from which translations are obtained
+        :param msgid:
+            Identifier of the message to translate
+        :returns:
+            Translated message or msgid if translation is not available
+        """
+
+    @abstractmethod
+    def pdngettext(self, msgctxt, domain, msgid1, msgid2, n):
+        """
+        Translate a message involving plural form using a specific context and
+        domain
+
+        :param msgctxt:
+            Context that specifies which translation of msgid1/msgid2 to pick
+        :param domain:
+            Name of the domain from which translations are obtained
+        :param msgid1:
+            Identifier of the singular form of the message to translate
+        :param msgid2:
+            Identifier of the plural form of message to translate
+        :param n:
+            Any integer number
+        :returns:
+            Translated message appropriate for the specified number, if
+            available.  If the translated number is not available one of msgid1
+            and msgid2 are returned, depending on the value of n.
+        """
