@@ -112,17 +112,17 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         })
         # Titles + borders
         self.format04 = self.workbook.add_format({
-            'align': 'left', 'size': 12, 'bold': 1, 'border': 7
+            'align': 'left', 'size': 12, 'bold': 1, 'border': 1
         })
         # System info with borders
         self.format05 = self.workbook.add_format({
             'align': 'left', 'valign': 'vcenter', 'text_wrap': 1, 'size': 8,
-            'border': 7,
+            'border': 1,
         })
         # System info with borders, grayed out background
         self.format06 = self.workbook.add_format({
             'align': 'left', 'valign': 'vcenter', 'text_wrap': 1, 'size': 8,
-            'border': 7, 'bg_color': '#E6E6E6',
+            'border': 1, 'bg_color': '#E6E6E6',
         })
         # Headlines (center)
         self.format07 = self.workbook.add_format({
@@ -140,17 +140,17 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         # Green background / Size 8
         self.format10 = self.workbook.add_format({
             'align': 'center', 'valign': 'vcenter', 'text_wrap': 1, 'size': 8,
-            'bg_color': 'lime', 'border': 7, 'border_color': 'white',
+            'bg_color': 'lime', 'border': 1, 'border_color': 'white',
         })
         # Red background / Size 8
         self.format11 = self.workbook.add_format({
             'align': 'center', 'valign': 'vcenter', 'text_wrap': 1, 'size': 8,
-            'bg_color': 'red', 'border': 7, 'border_color': 'white',
+            'bg_color': 'red', 'border': 1, 'border_color': 'white',
         })
         # Gray background / Size 8
         self.format12 = self.workbook.add_format({
             'align': 'center', 'valign': 'vcenter', 'text_wrap': 1, 'size': 8,
-            'bg_color': 'gray', 'border': 7, 'border_color': 'white',
+            'bg_color': 'gray', 'border': 1, 'border_color': 'white',
         })
         # Attachments
         self.format13 = self.workbook.add_format({
@@ -176,14 +176,16 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
     def _hw_collection(self, data):
         hw_info = defaultdict(lambda: 'NA')
         if '2013.com.canonical.certification::dmi' in data['resource_map']:
-            result = ['{} {} ({})'.format(i['vendor'], i['product'],
-                      i['version']) for i in data["resource_map"]['2013.com.canonical.certification::dmi']
-                      if i['category'] == 'SYSTEM']
+            result = [
+                '{} {} ({})'.format(i.get('vendor'), i.get('product'), i.get('version'))
+                for i in data["resource_map"]['2013.com.canonical.certification::dmi']
+                if i.get('category') == 'SYSTEM']
             if result:
                 hw_info['platform'] = result.pop()
-            result = ['{}'.format(i['version'])
-                      for i in data["resource_map"]['2013.com.canonical.certification::dmi']
-                      if i['category'] == 'BIOS']
+            result = [
+                '{}'.format(i.get('version'))
+                for i in data["resource_map"]['2013.com.canonical.certification::dmi']
+                if i.get('category') == 'BIOS']
             if result:
                 hw_info['bios'] = result.pop()
         if '2013.com.canonical.certification::cpuinfo' in data['resource_map']:
@@ -227,15 +229,56 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                       '.1f')) for i in data["resource_map"]['2013.com.canonical.certification::meminfo']]
             if result:
                 hw_info['memory'] = result.pop()
-        if '2013.com.canonical.certification::device' in data['resource_map']:
-            result = ['{}'.format(i['product'])
-                      for i in data["resource_map"]['2013.com.canonical.certification::device']
-                      if ('category' in i and i['category'] == 'BLUETOOTH' and
-                          'driver' in i)]
-            if result:
-                hw_info['bluetooth'] = result.pop()
+        bluetooth = self._get_bluetooth_product_or_path(data)
+        if bluetooth:
+                hw_info['bluetooth'] = bluetooth
         return hw_info
 
+    def _get_resource_list(self, data, resource_id):
+        """
+        Get a list of resource objects associated with the specified job
+        (resource) identifier
+
+        :param data:
+            Exporter data
+        :param resource_id:
+            Identifier of the job / resource
+        :returns:
+            A list of matching resource objects. If there are no resources of
+            that kind then an empty list list returned.
+        """
+        resource_map = data.get('resource_map')
+        if not resource_map:
+            return []
+        return resource_map.get(resource_id, [])
+
+    def _get_bluetooth_product_or_path(
+            self, data,
+            resource_id='2013.com.canonical.certification::device'):
+        """
+        Get the 'product' or 'path' of the first bluetooth device.
+
+        :param data:
+            Exporter data
+        :param resource_id:
+            (optional) Identifier of the device resource.
+        :returns:
+            The 'product' attribute, or the 'path' attribute or None if no such
+            device can be found.
+
+        This method finds the name of the 'product' or 'path' attributes (first
+        available one wins) associated with a resource that has a 'category'
+        attribute equal to BLUETOOTH. The resource is looked using the supplied
+        (default) resource identifier.
+        """
+        for resource in self._get_resource_list(data, resource_id):
+            if resource.get('category') != 'BLUETOOTH':
+                continue
+            if 'product' in resource:
+                return resource['product']
+            if 'path' in resource:
+                return resource['path']
+
     def write_systeminfo(self, data):
         self.worksheet1.set_column(0, 0, 4)
         self.worksheet1.set_column(1, 1, 34)
@@ -293,6 +336,14 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
             )
 
     def write_summary(self, data):
+        if self.total != 0:
+            pass_rate = "{:.2f}%".format(self.total_pass / self.total * 100)
+            fail_rate = "{:.2f}%".format(self.total_fail / self.total * 100)
+            skip_rate = "{:.2f}%".format(self.total_skip / self.total * 100)
+        else:
+            pass_rate = _("N/A")
+            fail_rate = _("N/A")
+            skip_rate = _("N/A")
         self.worksheet2.set_column(0, 0, 5)
         self.worksheet2.set_column(1, 1, 2)
         self.worksheet2.set_column(3, 3, 27)
@@ -303,28 +354,26 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 ngettext('{} Test passed', '{} Tests passed',
                          self.total_pass).format(self.total_pass)
                 + " - "
-                + _('Success Rate: {:.2f}% ({}/{})').format(
-                    self.total_pass / self.total * 100,
-                    self.total_pass, self.total)
+                + _('Success Rate: {} ({}/{})').format(
+                    pass_rate, self.total_pass, self.total)
             ), self.format02)
         self.worksheet2.write(5, 1, '✘', self.format11)
         self.worksheet2.write(
             5, 2, (
-                ngettext('{} Test failed', '{} Tests failed', self.total_fail)
+                ngettext('{} Test failed', '{} Tests failed',
+                         self.total_fail).format(self.total_fail)
                 + ' - '
-                + _('Failure Rate: {:.2f}% ({}/{})').format(
-                    self.total_fail / self.total * 100,
-                    self.total_fail, self.total)
+                + _('Failure Rate: {} ({}/{})').format(
+                    fail_rate, self.total_fail, self.total)
             ), self.format02)
         self.worksheet2.write(6, 1, '-', self.format12)
         self.worksheet2.write(
             6, 2, (
                 ngettext('{} Test skipped', '{} Tests skipped',
-                         self.total_skip)
+                         self.total_skip).format(self.total_skip)
                 + ' - '
-                + _('Skip Rate: {:.2f}% ({}/{})').format(
-                    self.total_skip / self.total * 100,
-                    self.total_skip, self.total)
+                + _('Skip Rate: {} ({}/{})').format(
+                    skip_rate, self.total_skip, self.total)
             ), self.format02)
         self.worksheet2.write_column(
             'L3', [_('Fail'), _('Skip'), _('Pass')], self.format14)
@@ -345,7 +394,7 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         )
         # Insert the chart into the worksheet.
         self.worksheet2.insert_chart('F4', chart, {
-            'x_offset': 0, 'y_offset': 10, 'x_scale': 0.25, 'y_scale': 0.25
+            'x_offset': 0, 'y_offset': 10, 'x_scale': 0.50, 'y_scale': 0.50
         })
 
     def _set_category_status(self, result_map, via, child):
@@ -355,6 +404,9 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
             child_status = result_map[child]['outcome']
             if 'category_status' in result_map[child]:
                 child_status = result_map[child]['category_status']
+            # Ignore categories without any child
+            elif result_map[child]['plugin'] == 'local':
+                continue
             if child_status == IJobResult.OUTCOME_FAIL:
                 result_map[parent]['category_status'] = IJobResult.OUTCOME_FAIL
             elif (
@@ -393,11 +445,14 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 sorted(
                     tree.items(),
                     key=lambda t: 'z' + t[0] if t[1] else 'a' + t[0])).items():
+            if (result_map[job]['plugin'] == 'local' and
+                not result_map[job].get('category_status')):
+                continue
             self._lineno += 1
             if children:
                 self.worksheet3.write(
                     self._lineno, level + 1,
-                    result_map[job]['description'], self.format15)
+                    result_map[job]['summary'], self.format15)
                 if (
                     result_map[job]['category_status'] ==
                     IJobResult.OUTCOME_PASS
@@ -419,7 +474,7 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 if self.OPTION_WITH_DESCRIPTION in self._option_list:
                     self.worksheet4.write(
                         self._lineno, level + 1,
-                        result_map[job]['description'], self.format15)
+                        result_map[job].get('description', ""), self.format15)
                 if level:
                     self.worksheet3.set_row(
                         self._lineno, 13, None, {'level': level})
@@ -433,7 +488,7 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 self._write_job(children, result_map, max_level, level + 1)
             else:
                 self.worksheet3.write(
-                    self._lineno, max_level + 1, job,
+                    self._lineno, max_level + 1, result_map[job]['summary'],
                     self.format08 if self._lineno % 2 else self.format09)
                 if self.OPTION_WITH_DESCRIPTION in self._option_list:
                     link_cell = xl_rowcol_to_cell(self._lineno, max_level + 1)
@@ -441,9 +496,10 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                         self._lineno, max_level + 1,
                         'internal:' + _("Test Descriptions") + '!' + link_cell,
                         self.format08 if self._lineno % 2 else self.format09,
-                        job)
+                        result_map[job]['summary'])
                     self.worksheet4.write(
-                        self._lineno, max_level + 1, job,
+                        self._lineno, max_level + 1,
+                        result_map[job]['summary'],
                         self.format08 if self._lineno % 2 else self.format09)
                 self.total += 1
                 if result_map[job]['outcome'] == IJobResult.OUTCOME_PASS:
@@ -484,7 +540,8 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                         result_map[job]['io_log'].encode()
                     ).decode('UTF-8').rstrip()
                 io_lines = len(io_log.splitlines()) - 1
-                desc_lines = len(result_map[job]['description'].splitlines())
+                desc_lines = len(result_map[job].get('description',
+                                                     "").splitlines())
                 desc_lines -= 1
                 self.worksheet3.write(
                     self._lineno, max_level + 3, io_log,
@@ -492,21 +549,21 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
                 if self.OPTION_WITH_DESCRIPTION in self._option_list:
                     self.worksheet4.write(
                         self._lineno, max_level + 2,
-                        result_map[job]['description'],
+                        result_map[job].get('description', ""),
                         self.format16 if self._lineno % 2 else self.format17)
                 if level:
                     self.worksheet3.set_row(
-                        self._lineno, 12 + 9.71 * io_lines,
+                        self._lineno, 12 + 10.5 * io_lines,
                         None, {'level': level})
                     if self.OPTION_WITH_DESCRIPTION in self._option_list:
                         self.worksheet4.set_row(
-                            self._lineno, 12 + 9.71 * desc_lines,
+                            self._lineno, 12 + 10.5 * desc_lines,
                             None, {'level': level})
                 else:
-                    self.worksheet3.set_row(self._lineno, 12 + 9.71 * io_lines)
+                    self.worksheet3.set_row(self._lineno, 12 + 10.5 * io_lines)
                     if self.OPTION_WITH_DESCRIPTION in self._option_list:
                         self.worksheet4.set_row(
-                            self._lineno, 12 + 9.71 * desc_lines)
+                            self._lineno, 12 + 10.5 * desc_lines)
 
     def write_results(self, data):
         tree, max_level = self._tree(data['result_map'])
@@ -515,9 +572,9 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
         self.worksheet3.set_tab_color('#DC4C00')  # Orange
         self.worksheet3.set_column(0, 0, 5)
         [self.worksheet3.set_column(i, i, 2) for i in range(1, max_level + 1)]
-        self.worksheet3.set_column(max_level + 1, max_level + 0, 48)
-        self.worksheet3.set_column(max_level + 2, max_level + 1, 12)
-        self.worksheet3.set_column(max_level + 3, max_level + 2, 65)
+        self.worksheet3.set_column(max_level + 1, max_level + 1, 48)
+        self.worksheet3.set_column(max_level + 2, max_level + 2, 12)
+        self.worksheet3.set_column(max_level + 3, max_level + 3, 65)
         self.worksheet3.write_row(
             5, max_level + 1, [_('Name'), _('Result'), _('I/O Log')],
             self.format07)
@@ -527,8 +584,8 @@ class XLSXSessionStateExporter(SessionStateExporterBase):
             self.worksheet4.set_column(0, 0, 5)
             [self.worksheet4.set_column(i, i, 2)
                 for i in range(1, max_level + 1)]
-            self.worksheet4.set_column(max_level + 1, max_level + 0, 48)
-            self.worksheet4.set_column(max_level + 2, max_level + 1, 65)
+            self.worksheet4.set_column(max_level + 1, max_level + 1, 48)
+            self.worksheet4.set_column(max_level + 2, max_level + 2, 65)
             self.worksheet4.write_row(
                 5, max_level + 1, [_('Name'), _('Description')], self.format07
             )
