@@ -115,6 +115,10 @@ class DummyProvider1(IProvider1, IProviderBackend1):
     def base_dir(self):
         return self._extras.get("base_dir")
 
+    @property
+    def units_dir(self):
+        return self._extras.get("units_dir")
+
     def secure(self):
         return False
 
@@ -173,13 +177,13 @@ class InsecureProvider1PlugInCollection(FsPlugInCollection):
     PATH, but is used for looking up providers.
     """
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         PROVIDERPATH = os.getenv("PROVIDERPATH")
         if PROVIDERPATH is None:
             dir_list = get_insecure_PROVIDERPATH_list()
         else:
             dir_list = PROVIDERPATH.split(os.path.pathsep)
-        super().__init__(dir_list, '.provider', wrapper=Provider1PlugIn)
+        super().__init__(dir_list, '.provider', wrapper=Provider1PlugIn, **kwargs)
 
 
 # Collection of all providers
