@@ -29,197 +29,18 @@ Implementation of rfc822 serializer and deserializer.
     THIS MODULE DOES NOT HAVE STABLE PUBLIC API
 """
 
-from functools import total_ordering
-import inspect
 import logging
-import os
 import re
 import textwrap
 
-from plainbox.abc import ITextSource
 from plainbox.i18n import gettext as _
+from plainbox.impl.secure.origin import FileTextSource
+from plainbox.impl.secure.origin import Origin
+from plainbox.impl.secure.origin import UnknownTextSource
 
 logger = logging.getLogger("plainbox.secure.rfc822")
 
 
-@total_ordering
-class UnknownTextSource(ITextSource):
-    """
-    A :class:`ITextSource` subclass indicating that the source of text is
-    unknown.
-
-    This instances of this class are constructed by gen_rfc822_records() when
-    no explicit source is provided and the stream has no name. The serve as
-    non-None values to prevent constructing :class:`PythonFileTextSource` with
-    origin computed from :meth:`Origin.get_caller_origin()`
-    """
-
-    def __str__(self):
-        return _("???")
-
-    def __repr__(self):
-        return "{}()".format(self.__class__.__name__)
-
-    def __eq__(self, other):
-        if isinstance(other, UnknownTextSource):
-            return True
-        else:
-            return False
-
-    def __gt__(self, other):
-        if isinstance(other, UnknownTextSource):
-            return False
-        else:
-            return NotImplemented
-
-
-@total_ordering
-class FileTextSource(ITextSource):
-    """
-    A :class:`ITextSource` subclass indicating that text came from a file.
-
-    :ivar filename:
-        name of the file something comes from
-    """
-
-    def __init__(self, filename):
-        self.filename = filename
-
-    def __str__(self):
-        return self.filename
-
-    def __repr__(self):
-        return "{}({!r})".format(
-            self.__class__.__name__, self.filename)
-
-    def __eq__(self, other):
-        if isinstance(other, FileTextSource):
-            return self.filename == other.filename
-        else:
-            return False
-
-    def __gt__(self, other):
-        if isinstance(other, FileTextSource):
-            return self.filename > other.filename
-        else:
-            return NotImplemented
-
-    def relative_to(self, base_dir):
-        """
-        Compute a FileTextSource with the filename being a realtive path from
-        the specified base directory.
-
-        :param base_dir:
-            A base directory name
-        :returns:
-            A new FileTextSource with filename relative to that base_dir
-        """
-        return self.__class__(os.path.relpath(self.filename, base_dir))
-
-
-class PythonFileTextSource(FileTextSource):
-    """
-    A :class:`FileTextSource` subclass indicating the file was a python file.
-
-    It implements no differences but in some context it might be helpful to
-    differentiate on the type of the source field in the origin of a job
-    definition record.
-
-    :ivar filename:
-        name of the python filename that something comes from
-    """
-
-
-@total_ordering
-class Origin:
-    """
-    Simple class for tracking where something came from
-
-    :ivar source:
-        something that describes where the text came frome, technically it
-        should be a :class:`~plainbox.abc.ITextSource` subclass but that
-        interface defines just the intent, not any concrete API.
-
-    :ivar line_start:
-        the number of the line where the record begins
-
-    :ivar line_end:
-        the number of the line where the record ends
-    """
-
-    __slots__ = ['source', 'line_start', 'line_end']
-
-    def __init__(self, source, line_start, line_end):
-        self.source = source
-        self.line_start = line_start
-        self.line_end = line_end
-
-    def __repr__(self):
-        return "<{} source:{!r} line_start:{} line_end:{}>".format(
-            self.__class__.__name__,
-            self.source, self.line_start, self.line_end)
-
-    def __str__(self):
-        return "{}:{}-{}".format(
-            self.source, self.line_start, self.line_end)
-
-    def relative_to(self, base_dir):
-        """
-        Create a Origin with source relative to the specified base directory.
-
-        :param base_dir:
-            A base directory name
-        :returns:
-            A new Origin with source replaced by the result of calling
-            relative_to(base_dir) on the current source *iff* the current
-            source has that method, self otherwise.
-
-        This method is useful for obtaining user friendly Origin objects that
-        have short, understandable filenames.
-        """
-        if hasattr(self.source, 'relative_to'):
-            return Origin(
-                self.source.relative_to(base_dir),
-                self.line_start, self.line_end)
-        else:
-            return self
-
-    def __eq__(self, other):
-        if isinstance(other, Origin):
-            return ((self.source, self.line_start, self.line_end) ==
-                    (other.source, other.line_start, other.line_end))
-        else:
-            return NotImplemented
-
-    def __gt__(self, other):
-        if isinstance(other, Origin):
-            return ((self.source, self.line_start, self.line_end) >
-                    (other.source, other.line_start, other.line_end))
-        else:
-            return NotImplemented
-
-    @classmethod
-    def get_caller_origin(cls, back=0):
-        """
-        Create an Origin instance pointing at the call site of this method.
-        """
-        # Create an Origin instance that pinpoints the place that called
-        # get_caller_origin().
-        caller_frame, filename, lineno = inspect.stack(0)[2 + back][:3]
-        try:
-            source = PythonFileTextSource(filename)
-            origin = Origin(source, lineno, lineno)
-        finally:
-            # Explicitly delete the frame object, this breaks the
-            # reference cycle and makes this part of the code deterministic
-            # with regards to the CPython garbage collector.
-            #
-            # As recommended by the python documentation:
-            # http://docs.python.org/3/library/inspect.html#the-interpreter-stack
-            del caller_frame
-        return origin
-
-
 def normalize_rfc822_value(value):
     # Remove the multi-line dot marker
     value = re.sub('^(\s*)\.$', '\\1', value, flags=re.M)
@@ -246,7 +67,7 @@ class RFC822Record:
     file/stream where it was parsed from).
     """
 
-    def __init__(self, data, origin=None, raw_data=None):
+    def __init__(self, data, origin=None, raw_data=None, field_offset_map=None):
         """
         Initialize a new record.
 
@@ -258,6 +79,8 @@ class RFC822Record:
             An optional dictionary with raw record data. If omitted then it
             will default to normalized data (as the same object, without making
             a copy)
+        :param field_offset_map:
+            An optional dictionary with offsets (in line numbers) of each field
         """
         self._data = data
         if raw_data is None:
@@ -266,6 +89,7 @@ class RFC822Record:
         if origin is None:
             origin = Origin.get_caller_origin()
         self._origin = origin
+        self._field_offset_map = field_offset_map
 
     def __repr__(self):
         return "<{} data:{!r} origin:{!r}>".format(
@@ -313,19 +137,31 @@ class RFC822Record:
         """
         return self._origin
 
+    @property
+    def field_offset_map(self):
+        """
+        The field-to-line-number-offset mapping.
+
+        A dictionary mapping field name to offset (in lines) relative to the
+        origin where that field definition commences.
+
+        Note: the return value may be None
+        """
+        return self._field_offset_map
+
     def dump(self, stream):
         """
         Dump this record to a stream
         """
         def _dump_part(stream, key, values):
-            stream.write("%s:\n" % key)
+            stream.write("{}:\n".format(key))
             for value in values:
                 if not value:
                     stream.write(" .\n")
                 elif value == ".":
                     stream.write(" ..\n")
                 else:
-                    stream.write(" %s\n" % value)
+                    stream.write(" {}\n".format(value))
         for key, value in self.data.items():
             if isinstance(value, (list, tuple)):
                 _dump_part(stream, key, value)
@@ -335,7 +171,7 @@ class RFC822Record:
                     values = values[:-1]
                 _dump_part(stream, key, values)
             else:
-                stream.write("%s: %s\n" % (key, value))
+                stream.write("{}: {}\n".format(key, value))
         stream.write("\n")
 
 
@@ -426,6 +262,7 @@ def gen_rfc822_records(stream, data_cls=dict, source=None):
     key = None
     value_list = None
     origin = None
+    field_offset_map = None
     # If the source was not provided then try constructing a FileTextSource
     # from the name of the stream. If that fails, keep using None.
     if source is None:
@@ -452,11 +289,13 @@ def gen_rfc822_records(stream, data_cls=dict, source=None):
         nonlocal value_list
         nonlocal record
         nonlocal origin
+        nonlocal field_offset_map
         key = None
         value_list = None
         if source is not None:
             origin = Origin(source, None, None)
-        record = RFC822Record(data_cls(), origin, data_cls())
+        field_offset_map = {}
+        record = RFC822Record(data_cls(), origin, data_cls(), field_offset_map)
 
     def _commit_key_value_if_needed():
         """
@@ -550,10 +389,19 @@ def gen_rfc822_records(stream, data_cls=dict, source=None):
                 # have so far. Additional multi-line values will just append to
                 # value_list
                 value_list = [value]
+                # Store the offset of the filed in the offset map
+                field_offset_map[key] = lineno - origin.line_start
             else:
                 # The initial line may be empty, in that case the spaces and
                 # newlines there are discarded
                 value_list = []
+                # Store the offset of the filed in the offset map
+                # The +1 is for the fact that value is empty (or just
+                # whitespace) and that is stripped away in the normalized data
+                # part of the RFC822 record. To keep line tracking accurate
+                # we just assume that the field actually starts on the following
+                # line.
+                field_offset_map[key] = lineno - origin.line_start + 1
             # Update the end-line location
             _update_end_lineno()
         # Treat all other lines as syntax errors
