@@ -8,7 +8,6 @@
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
 # as published by the Free Software Foundation.
-
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,106 +28,32 @@ import re
 from plainbox.abc import IJobDefinition
 from plainbox.i18n import gettext as _
 from plainbox.impl.resource import ResourceProgram
+from plainbox.impl.resource import parse_imports_stmt
 from plainbox.impl.secure.origin import JobOutputTextSource
 from plainbox.impl.secure.origin import Origin
 from plainbox.impl.symbol import SymbolDef
-from plainbox.impl.unit import Unit
+from plainbox.impl.unit._legacy import JobDefinitionLegacyAPI
+from plainbox.impl.unit.unit_with_id import UnitWithId
+from plainbox.impl.unit.validators import CorrectFieldValueValidator
+from plainbox.impl.unit.validators import DeprecatedFieldValidator
+from plainbox.impl.unit.validators import PresentFieldValidator
+from plainbox.impl.unit.validators import ReferenceConstraint
+from plainbox.impl.unit.validators import ShellProgramValidator
+from plainbox.impl.unit.validators import TemplateInvariantFieldValidator
+from plainbox.impl.unit.validators import TemplateVariantFieldValidator
+from plainbox.impl.unit.validators import TranslatableFieldValidator
+from plainbox.impl.unit.validators import UnitReferenceValidator
+from plainbox.impl.unit.validators import UntranslatableFieldValidator
+from plainbox.impl.unit.validators import UselessFieldValidator
 from plainbox.impl.validation import Problem
-from plainbox.impl.validation import ValidationError
-from plainbox.impl.resource import parse_imports_stmt
-from plainbox.impl.resource import ResourceProgramError
+from plainbox.impl.validation import Severity
+
+__all__ = ['JobDefinition', 'propertywithsymbols']
 
 
 logger = logging.getLogger("plainbox.unit.job")
 
 
-class JobDefinitionValidator:
-    """
-    Validator for JobDefinition units.
-    """
-
-    @staticmethod
-    def validate(job, strict=False, deprecated=False):
-        """
-        Validate the specified job
-
-        :param strict:
-            Enforce strict validation. Non-conforming jobs will be rejected.
-            This is off by default to ensure that non-critical errors don't
-            prevent jobs from running.
-        :param deprecated:
-            Enforce deprecation validation. Jobs having deprecated fields will
-            be rejected. This is off by default to allow backwards compatible
-            jobs to be used without any changes.
-        """
-        # Check if name is still being used, if running in strict mode
-        if deprecated and job.name is not None:
-            raise ValidationError(job.fields.name, Problem.deprecated)
-        # Check if the partial_id field is empty
-        if job.partial_id is None:
-            raise ValidationError(job.fields.id, Problem.missing)
-        # Check if summary is empty, if running in strict mode
-        if strict and job.summary is None:
-            raise ValidationError(job.fields.summary, Problem.missing)
-        # Check if plugin is empty
-        if job.plugin is None:
-            raise ValidationError(job.fields.plugin, Problem.missing)
-        # Check if plugin has a good value
-        if job.plugin not in JobDefinition.plugin.get_all_symbols():
-            raise ValidationError(job.fields.plugin, Problem.wrong)
-        # Check if user is given without a command to run, if running in strict
-        # mode
-        if strict and job.user is not None and job.command is None:
-            raise ValidationError(job.fields.user, Problem.useless)
-        # Check if environ is given without a command to run, if running in
-        # strict mode
-        if strict and job.environ is not None and job.command is None:
-            raise ValidationError(job.fields.environ, Problem.useless)
-        # Verify that command is present on a job within the subset that should
-        # really have them (shell, local, resource, attachment, user-verify and
-        # user-interact)
-        if job.plugin in {JobDefinition.plugin.shell,
-                          JobDefinition.plugin.local,
-                          JobDefinition.plugin.resource,
-                          JobDefinition.plugin.attachment,
-                          JobDefinition.plugin.user_verify,
-                          JobDefinition.plugin.user_interact,
-                          JobDefinition.plugin.user_interact_verify}:
-            # Check if shell jobs have a command
-            if job.command is None:
-                raise ValidationError(job.fields.command, Problem.missing)
-            # Check if user has a good value
-            if job.user not in (None, "root"):
-                raise ValidationError(job.fields.user, Problem.wrong)
-        # Do some special checks for manual jobs as those should really be
-        # fully interactive, non-automated jobs (otherwise they are either
-        # user-interact or user-verify)
-        if job.plugin == JobDefinition.plugin.manual:
-            # Ensure that manual jobs have a description
-            if job.description is None:
-                raise ValidationError(
-                    job.fields.description, Problem.missing)
-            # Ensure that manual jobs don't have command, if running in strict
-            # mode
-            if strict and job.command is not None:
-                raise ValidationError(job.fields.command, Problem.useless)
-        estimated_duration = job.get_record_value('estimated_duration')
-        if estimated_duration is not None:
-            try:
-                float(estimated_duration)
-            except ValueError:
-                raise ValidationError(
-                    job.fields.estimated_duration, Problem.wrong)
-        elif strict and estimated_duration is None:
-            raise ValidationError(
-                job.fields.estimated_duration, Problem.missing)
-        # The resource program should be valid
-        try:
-            job.get_resource_program()
-        except ResourceProgramError:
-            raise ValidationError(job.fields.requires, Problem.wrong)
-
-
 class propertywithsymbols(property):
     """
     A property that also keeps a group of symbols around
@@ -164,7 +89,21 @@ class propertywithsymbols(property):
             fget, self.fset, self.fdel, self.__doc__, symbols=self.symbols)
 
 
-class JobDefinition(Unit, IJobDefinition):
+class _PluginValues(SymbolDef):
+    """
+    Symbols for each value of the JobDefinition.plugin field
+    """
+    attachment = 'attachment'
+    local = 'local'
+    resource = 'resource'
+    manual = 'manual'
+    user_verify = "user-verify"
+    user_interact = "user-interact"
+    user_interact_verify = "user-interact-verify"
+    shell = 'shell'
+
+
+class JobDefinition(UnitWithId, JobDefinitionLegacyAPI, IJobDefinition):
     """
     Job definition class.
 
@@ -173,7 +112,7 @@ class JobDefinition(Unit, IJobDefinition):
     """
 
     def __init__(self, data, origin=None, provider=None, controller=None,
-                 raw_data=None, parameters=None):
+                 raw_data=None, parameters=None, field_offset_map=None):
         """
         Initialize a new JobDefinition instance.
 
@@ -200,6 +139,9 @@ class JobDefinition(Unit, IJobDefinition):
             This is required to obtain translated summary and description
             fields, while having a single translated base text and any
             variation in the available parameters.
+        :param field_offset_map:
+            An optional dictionary with offsets (in line numbers) of each
+            field.  Line numbers are relative to the value of origin.line_start
 
         .. note::
             You should almost always use :meth:`from_rfc822_record()` instead.
@@ -207,7 +149,8 @@ class JobDefinition(Unit, IJobDefinition):
         if origin is None:
             origin = Origin.get_caller_origin()
         super().__init__(data, raw_data=raw_data, origin=origin,
-                         provider=provider, parameters=parameters)
+                         provider=provider, parameters=parameters,
+                         field_offset_map=field_offset_map)
         # NOTE: controllers cannot be customized for instantiated templates so
         # I wonder if we should start hard-coding it in. Nothing seems to be
         # using custom controller functionality anymore.
@@ -220,7 +163,7 @@ class JobDefinition(Unit, IJobDefinition):
 
     @classmethod
     def instantiate_template(cls, data, raw_data, origin, provider,
-                             parameters):
+                             parameters, field_offset_map):
         """
         Instantiate this unit from a template.
 
@@ -235,7 +178,8 @@ class JobDefinition(Unit, IJobDefinition):
         # called with correctly-ordered arguments.
         assert cls is JobDefinition, \
             "{}.instantiate_template() not customized".format(cls.__name__)
-        return cls(data, origin, provider, None, raw_data, parameters)
+        return cls(data, origin, provider, None, raw_data, parameters,
+                   field_offset_map)
 
     def __str__(self):
         return self.summary
@@ -244,81 +188,6 @@ class JobDefinition(Unit, IJobDefinition):
         return "<JobDefinition id:{!r} plugin:{!r}>".format(
             self.id, self.plugin)
 
-    class fields(SymbolDef):
-        """
-        Symbols for each field that a JobDefinition can have
-        """
-        name = 'name'
-        id = 'id'
-        summary = 'summary'
-        plugin = 'plugin'
-        command = 'command'
-        description = 'description'
-        user = 'user'
-        environ = 'environ'
-        estimated_duration = 'estimated_duration'
-        depends = 'depends'
-        requires = 'requires'
-        shell = 'shell'
-        imports = 'imports'
-        category_id = 'category_id'
-
-    class Meta(Unit.Meta):
-
-        template_constraints = {
-            'name': 'vary',
-            'unit': 'const',
-            # The 'id' field should be always variable (depending on at least
-            # resource reference) or clashes are inevitable (they can *still*
-            # occur but this is something we cannot prevent).
-            'id': 'vary',
-            # The summary should never be constant as that would be confusing
-            # to the test operator. If it is defined in the template it should
-            # be customized by at least one resource reference.
-            'summary': 'vary',
-            # The 'plugin' field should be constant as otherwise validation is
-            # very unreliable. There is no current demand for being able to
-            # customize it from a resource record.
-            'plugin': 'const',
-            # The command field should be variable if it is defined. This may
-            # be too strict but there has to be a channel between the resource
-            # object and the command to make the template job valuable so for
-            # now we will require variability here.
-            'command': 'vary',
-            # The description should never be constant as that would be
-            # confusing to the test operator. If it is defined in the template
-            # it should be customized by at least one resource reference.
-            'description': 'vary',
-            # There is no conceivable value in having a variable user field
-            'user': 'const',
-            'environ': 'const',
-            # TODO: what about estimated duration?
-            # 'estimated_duration': '?',
-            # TODO: what about depends and requires?
-            #
-            # If both are const then we can determine test ordering without any
-            # action and the ordering is not perturbed at runtime. This may be
-            # too strong of a limitation though. We'll see.
-            # 'depends': '?',
-            # 'requires': '?',
-            'shell': 'const',
-            'imports': 'const',
-            'category_id': 'const',
-        }
-
-    class _PluginValues(SymbolDef):
-        """
-        Symbols for each value of the JobDefinition.plugin field
-        """
-        attachment = 'attachment'
-        local = 'local'
-        resource = 'resource'
-        manual = 'manual'
-        user_verify = "user-verify"
-        user_interact = "user-interact"
-        user_interact_verify = "user-interact-verify"
-        shell = 'shell'
-
     @property
     def unit(self):
         """
@@ -336,9 +205,6 @@ class JobDefinition(Unit, IJobDefinition):
         """
         return _("job")
 
-    def get_unit_type(self):
-        return _("job")
-
     @property
     def partial_id(self):
         """
@@ -346,18 +212,7 @@ class JobDefinition(Unit, IJobDefinition):
 
         This field should not be used anymore, except for display
         """
-        return self.get_record_value('id', self.name)
-
-    @property
-    def id(self):
-        if self._provider:
-            return "{}::{}".format(self._provider.namespace, self.partial_id)
-        else:
-            return self.partial_id
-
-    @property
-    def name(self):
-        return self.get_record_value('name')
+        return self.get_record_value('id', self.get_record_value('name'))
 
     @propertywithsymbols(symbols=_PluginValues)
     def plugin(self):
@@ -422,16 +277,11 @@ class JobDefinition(Unit, IJobDefinition):
         second.
         """
         value = self.get_record_value('estimated_duration')
-        # TODO: make the error case case detected by job validator
-        if value is None:
-            return
-        try:
-            return float(value)
-        except ValueError:
-            # TRANSLATORS: keep "estimated_duration" untranslated.
-            logger.warning(
-                _("Incorrect value of 'estimated_duration' in job"
-                  " %s read from %s"), self.id, self.origin)
+        if value is not None:
+            try:
+                return float(value)
+            except ValueError:
+                pass
 
     @property
     def controller(self):
@@ -562,14 +412,9 @@ class JobDefinition(Unit, IJobDefinition):
         To combat a simple mistake where the jobs are space-delimited any
         mixture of white-space (including newlines) and commas are allowed.
         """
-        def transform_id(some_id):
-            if "::" not in some_id and self._provider is not None:
-                return "{}::{}".format(self._provider.namespace, some_id)
-            else:
-                return some_id
         if self.depends:
             return {
-                transform_id(maybe_partial_id)
+                self.qualify_id(maybe_partial_id)
                 for maybe_partial_id in re.split('[\s,]+', self.depends)
             }
         else:
@@ -585,6 +430,14 @@ class JobDefinition(Unit, IJobDefinition):
         else:
             return set()
 
+    def get_category_id(self):
+        """
+        Get the fully-qualified category id that this job belongs to
+        """
+        maybe_partial_id = self.category_id
+        if maybe_partial_id is not None:
+            return self.qualify_id(maybe_partial_id)
+
     @classmethod
     def from_rfc822_record(cls, record, provider=None):
         """
@@ -599,20 +452,8 @@ class JobDefinition(Unit, IJobDefinition):
         # (xgettext strips out those newlines)
         return cls(record.data, record.origin, provider=provider, raw_data={
             key: value.rstrip('\n')
-            for key, value in record.raw_data.items()})
-
-    def validate(self, **validation_kwargs):
-        """
-        Validate this job definition
-
-        :param validation_kwargs:
-            Keyword arguments to pass to the
-            :meth:`JobDefinitionValidator.validate()`
-        :raises ValidationError:
-            If the job has any problems that make it unsuitable for execution.
-        """
-        super().validate(**validation_kwargs)
-        JobDefinitionValidator.validate(self, **validation_kwargs)
+            for key, value in record.raw_data.items()
+        }, field_offset_map=record.field_offset_map)
 
     def create_child_job_from_record(self, record):
         """
@@ -629,3 +470,236 @@ class JobDefinition(Unit, IJobDefinition):
             # TRANSLATORS: don't translate record.origin.source.job
             raise ValueError(_("record.origin.source.job must be this job"))
         return self.from_rfc822_record(record, self.provider)
+
+    class Meta(UnitWithId.Meta, JobDefinitionLegacyAPI.Meta):
+
+        class fields(UnitWithId.Meta.fields):
+            """
+            Symbols for each field that a JobDefinition can have
+            """
+            name = 'name'
+            summary = 'summary'
+            plugin = 'plugin'
+            command = 'command'
+            description = 'description'
+            user = 'user'
+            environ = 'environ'
+            estimated_duration = 'estimated_duration'
+            depends = 'depends'
+            requires = 'requires'
+            shell = 'shell'
+            imports = 'imports'
+            flags = 'flags'
+            category_id = 'category_id'
+
+        field_validators = {}
+        field_validators.update(UnitWithId.Meta.field_validators)
+        field_validators.update({
+            fields.name: [
+                UntranslatableFieldValidator,
+                TemplateVariantFieldValidator,
+                DeprecatedFieldValidator(
+                    _("use 'id' and 'summary' instead of 'name'")),
+            ],
+            # NOTE: 'id' validators are "inherited" so we don't have it here
+            fields.summary: [
+                TranslatableFieldValidator,
+                TemplateVariantFieldValidator,
+                PresentFieldValidator(severity=Severity.advice),
+                # We want the summary to be a single line
+                CorrectFieldValueValidator(
+                    lambda summary: summary.count("\n") == 0,
+                    Problem.wrong, Severity.warning,
+                    message=_("please use only one line"),
+                    onlyif=lambda unit: unit.summary is not None),
+                # We want the summary to be relatively short
+                CorrectFieldValueValidator(
+                    lambda summary: len(summary) <= 80,
+                    Problem.wrong, Severity.warning,
+                    message=_("please stay under 80 characters"),
+                    onlyif=lambda unit: unit.summary is not None),
+            ],
+            fields.plugin: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                PresentFieldValidator,
+                CorrectFieldValueValidator(
+                    lambda plugin: (
+                        plugin in JobDefinition.plugin.get_all_symbols()),
+                    message=_('valid values are: {}').format(
+                        ', '.join(str(sym) for sym in sorted(
+                            _PluginValues.get_all_symbols())))),
+                CorrectFieldValueValidator(
+                    lambda plugin: plugin != 'local',
+                    Problem.deprecated, Severity.advice,
+                    message=_("please migrate to job templates, "
+                              "see plainbox-template-unit(7) for details")),
+                CorrectFieldValueValidator(
+                    lambda plugin: plugin != 'user-verify',
+                    Problem.deprecated, Severity.advice,
+                    message=_("please migrate to user-interact-verify")),
+            ],
+            fields.command: [
+                UntranslatableFieldValidator,
+                TemplateVariantFieldValidator,
+                # All jobs except for manual must have a command
+                PresentFieldValidator(
+                    message=_("command is mandatory for non-manual jobs"),
+                    onlyif=lambda unit: unit.plugin != 'manual'),
+                # Manual jobs cannot have a command
+                UselessFieldValidator(
+                    message=_("command on a manual job makes no sense"),
+                    onlyif=lambda unit: unit.plugin == 'manual'),
+                # We don't want to refer to CHECKBOX_SHARE anymore
+                CorrectFieldValueValidator(
+                    lambda command: "CHECKBOX_SHARE" not in command,
+                    Problem.deprecated, Severity.advice,
+                    message=_("please use PLAINBOX_PROVIDER_DATA"
+                              " instead of CHECKBOX_SHARE"),
+                    onlyif=lambda unit: unit.command is not None),
+                # We don't want to refer to CHECKBOX_DATA anymore
+                CorrectFieldValueValidator(
+                    lambda command: "CHECKBOX_DATA" not in command,
+                    Problem.deprecated, Severity.advice,
+                    message=_("please use PLAINBOX_SESSION_SHARE"
+                              " instead of CHECKBOX_DATA"),
+                    onlyif=lambda unit: unit.command is not None),
+                # We want to catch silly mistakes that shlex can detect
+                ShellProgramValidator,
+            ],
+            fields.description: [
+                TranslatableFieldValidator,
+                TemplateVariantFieldValidator,
+                # Description is mandatory for manual jobs
+                PresentFieldValidator(
+                    message=_("manual jobs must have a description"),
+                    onlyif=lambda unit: unit.plugin == 'manual'),
+                # Description is recommended for all other jobs
+                PresentFieldValidator(
+                    severity=Severity.advice,
+                    message=_("all jobs should have a description"),
+                    onlyif=lambda unit: unit.plugin != 'manual')
+            ],
+            fields.user: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                # User should be either None or 'root'
+                CorrectFieldValueValidator(
+                    message=_("user can only be 'root'"),
+                    correct_fn=lambda user: user in (None, 'root')),
+                # User is useless without a command to run
+                UselessFieldValidator(
+                    message=_("user without a command makes no sense"),
+                    onlyif=lambda unit: unit.command is None)
+            ],
+            fields.environ: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                # Environ is useless without a command to run
+                UselessFieldValidator(
+                    message=_("environ without a command makes no sense"),
+                    onlyif=lambda unit: unit.command is None),
+            ],
+            fields.estimated_duration: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                PresentFieldValidator(severity=Severity.advice),
+                CorrectFieldValueValidator(
+                    lambda duration: float(duration) > 0,
+                    message="value must be a positive number",
+                    onlyif=lambda unit: (
+                        unit.get_record_value('estimated_duration'))),
+            ],
+            fields.depends: [
+                UntranslatableFieldValidator,
+                CorrectFieldValueValidator(
+                    lambda value, unit: (
+                        unit.get_direct_dependencies() is not None)),
+                UnitReferenceValidator(
+                    lambda unit: unit.get_direct_dependencies(),
+                    constraints=[
+                        ReferenceConstraint(
+                            lambda referrer, referee: referee.unit == 'job',
+                            message=_("the referenced unit is not a job"))])
+                # TODO: should not refer to deprecated jobs,
+                #       onlyif job itself is not deprecated
+            ],
+            fields.requires: [
+                UntranslatableFieldValidator,
+                CorrectFieldValueValidator(
+                    lambda value, unit: unit.get_resource_program(),
+                    onlyif=lambda unit: unit.requires is not None),
+                UnitReferenceValidator(
+                    lambda unit: unit.get_resource_dependencies(),
+                    constraints=[
+                        ReferenceConstraint(
+                            lambda referrer, referee: referee.unit == 'job',
+                            message=_("the referenced unit is not a job")),
+                        ReferenceConstraint(
+                            lambda referrer, referee: (
+                                referee.plugin == 'resource'),
+                            onlyif=lambda referrer, referee: (
+                                referee.unit == 'job'),
+                            message=_(
+                                "the referenced job is not a resource job")),
+                    ]),
+                # TODO: should not refer to deprecated jobs,
+                #       onlyif job itself is not deprecated
+            ],
+            fields.shell: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                # Shell should be only '/bin/sh', or None (which gives bash)
+                CorrectFieldValueValidator(
+                    lambda shell: shell in ('/bin/sh', '/bin/bash', 'bash'),
+                    message=_("only /bin/sh and /bin/bash are allowed")),
+            ],
+            fields.imports: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                CorrectFieldValueValidator(
+                    lambda value, unit: (
+                        list(unit.get_imported_jobs()) is not None)),
+                UnitReferenceValidator(
+                    lambda unit: [
+                        job_id
+                        for job_id, identifier in unit.get_imported_jobs()],
+                    constraints=[
+                        ReferenceConstraint(
+                            lambda referrer, referee: referee.unit == 'job',
+                            message=_("the referenced unit is not a job"))]),
+                # TODO: should not refer to deprecated jobs,
+                #       onlyif job itself is not deprecated
+            ],
+            fields.category_id: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                UnitReferenceValidator(
+                    lambda unit: (
+                        [unit.get_category_id()] if unit.category_id else ()),
+                    constraints=[
+                        ReferenceConstraint(
+                            lambda referrer, referee: (
+                                referee.unit == 'category'),
+                            message=_(
+                                "the referenced unit is not a category"))]),
+                # TODO: should not refer to deprecated categories,
+                #       onlyif job itself is not deprecated
+            ],
+            fields.flags: [
+                UntranslatableFieldValidator,
+                TemplateInvariantFieldValidator,
+                CorrectFieldValueValidator(
+                    lambda value, unit: (
+                        'preserve-locale' in unit.get_flag_set()),
+                    Problem.expected_i18n, Severity.advice,
+                    message=_(
+                        'please ensure that the command supports'
+                        ' non-C locale then set the preserve-locale flag'
+                    ),
+                    onlyif=lambda unit: unit.command),
+            ]
+        })
+
+
+JobDefinition.fields = JobDefinition.Meta.fields
