@@ -28,6 +28,7 @@ therein.
 
 import argparse
 import inspect
+import itertools
 import logging
 import os
 import re
@@ -39,19 +40,25 @@ from plainbox import __version__ as version
 from plainbox.i18n import docstring
 from plainbox.i18n import gettext as _
 from plainbox.i18n import gettext_noop as N_
+from plainbox.i18n import ngettext
 from plainbox.impl.buildsystems import all_buildsystems
 from plainbox.impl.commands import ToolBase, CommandBase
 from plainbox.impl.job import JobDefinition
-from plainbox.impl.unit import UnitWithId
 from plainbox.impl.logging import setup_logging
+from plainbox.impl.providers.v1 import InsecureProvider1PlugInCollection
 from plainbox.impl.providers.v1 import get_user_PROVIDERPATH_entry
 from plainbox.impl.secure.config import Unset
 from plainbox.impl.secure.config import ValidationError \
     as ConfigValidationError
+from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.providers.v1 import Provider1
 from plainbox.impl.secure.providers.v1 import Provider1Definition
 from plainbox.impl.secure.rfc822 import RFC822SyntaxError
+from plainbox.impl.unit.unit_with_id import UnitWithId
+from plainbox.impl.unit.validators import UnitValidationContext
+from plainbox.impl.validation import Issue
 from plainbox.impl.validation import Problem
+from plainbox.impl.validation import Severity
 from plainbox.impl.validation import ValidationError as UnitValidationError
 
 __all__ = ['setup', 'manage_py_extension']
@@ -829,8 +836,8 @@ class InfoCommand(ManageCommand):
             # 1: pathname of the file the job is defined in
             print("\t" + _("{0} {1}, from {2}").format(
                 unit.get_unit_type(),
-                unit.id if isinstance(unit, (UnitWithId, JobDefinition)) \
-                    else unit,
+                unit.id if isinstance(unit, (UnitWithId, JobDefinition))
+                else unit,
                 unit.origin.relative_to(self.definition.location)))
         if problem_list:
             print("\t" + _("Some units could not be parsed correctly"))
@@ -929,6 +936,63 @@ class ValidateCommand(ManageCommand):
         group.add_argument(
             '-L', '--legacy', dest='deprecated', action='store_false',
             help=_("Support deprecated syntax and features"))
+        group.add_argument(
+            '-N', '--new-validation-core', action='store_true',
+            help=_("Use the new validation subsystem"))
+
+    def invoked(self, ns):
+        if ns.new_validation_core:
+            return self.invoked_new(ns)
+        else:
+            return self.invoked_old(ns)
+
+    def invoked_old(self, ns):
+        provider = self.get_provider()
+        unit_list, load_problem_list = self.get_unit_list(provider)
+        validation_problem_list = self.validate_units(unit_list, ns)
+        explain = {
+            Problem.missing: _("missing definition of required field"),
+            Problem.wrong: _("incorrect value supplied"),
+            Problem.useless: _("useless field in this context"),
+            Problem.deprecated: _("usage of deprecated field"),
+            Problem.constant: _("template field is constant"),
+            Problem.variable: _("template field is variable"),
+        }
+        for unit, error in validation_problem_list:
+            if isinstance(error, UnitValidationError):
+                origin = error.origin if error.origin else unit.origin
+                problem_text = explain[error.problem]
+                if error.hint is not None:
+                    problem_text = error.hint
+                elif isinstance(unit, JobDefinition):
+                    # If this is a "wrong value" problem then perhaps we can
+                    # suggest the set of acceptable values? Those may be stored
+                    # as $field.symbols, though as of this writing that is only
+                    # true for the 'plugin' field.
+                    field_prop = getattr(JobDefinition, str(error.field))
+                    if (error.problem == Problem.wrong
+                            and hasattr(field_prop, "get_all_symbols")):
+                        symbol_list = field_prop.get_all_symbols()
+                        problem_text = _("allowed values are: {0}").format(
+                            ', '.join(str(symbol) for symbol in symbol_list))
+                # TRANSLATORS: fields are as follows:
+                # 0: filename with job definition
+                # 1: unit type name
+                # 2: unit identifier
+                # 3: field name
+                # 4: explanation of the problem
+                print(_("{0}: {1} {2!a}, field {3!a}: {4}").format(
+                    origin.relative_to(self.definition.location),
+                    unit.get_unit_type(),
+                    unit.id if isinstance(unit, (UnitWithId, JobDefinition))
+                    else unit,
+                    str(error.field), problem_text))
+            else:
+                print(str(error))
+        if validation_problem_list or load_problem_list:
+            return 1
+        else:
+            print(_("The provider seems to be valid"))
 
     def get_unit_list(self, provider):
         unit_list, problem_list = provider.get_units()
@@ -953,49 +1017,76 @@ class ValidateCommand(ManageCommand):
                 problem_list.append((unit, exc))
         return problem_list
 
-    def invoked(self, ns):
+    def invoked_new(self, ns):
+        _logger.info(_("Loading provider..."))
         provider = self.get_provider()
-        unit_list, load_problem_list = self.get_unit_list(provider)
-        validation_problem_list = self.validate_units(unit_list, ns)
-        explain = {
-            Problem.missing: _("missing definition of required field"),
-            Problem.wrong: _("incorrect value supplied"),
-            Problem.useless: _("useless field in this context"),
-            Problem.deprecated: _("usage of deprecated field"),
-            Problem.constant: _("template field is constant"),
-            Problem.variable: _("template field is variable"),
-        }
-        for unit, error in validation_problem_list:
-            if isinstance(error, UnitValidationError):
-                # TRANSLATORS: fields are as follows:
-                # 0: filename with job definition
-                # 1: unit type name
-                # 2: unit identifier
-                # 3: field name
-                # 4: explanation of the problem
-                print(_("{0}: {1} {2!a}, field {3!a}: {4}").format(
-                    unit.origin.relative_to(self.definition.location),
-                    unit.get_unit_type(),
-                    unit.id if isinstance(unit, (UnitWithId, JobDefinition)) \
-                        else unit,
-                    str(error.field), explain[error.problem]))
-                # If this is a "wrong value" problem then perhaps we can
-                # suggest the set of acceptable values? Those may be stored as
-                # $field.symbols, though as of this writing that is only true
-                # for the 'plugin' field.
-                field_prop = getattr(JobDefinition, str(error.field))
-                if (error.problem == Problem.wrong
-                        and hasattr(field_prop, "get_all_symbols")):
-                    symbol_list = field_prop.get_all_symbols()
-                    print(_("allowed values are: {0}").format(
-                        ', '.join(str(symbol) for symbol in symbol_list)))
+        _logger.info(_("Loading other providers..."))
+        all_providers = InsecureProvider1PlugInCollection(
+            validate=False, check=False)
+        all_providers.load()
+        provider_list = all_providers.get_all_plugin_objects()
+        if all(p.name != provider.name for p in provider_list):
+            provider_list.append(provider)
+        _logger.info(_("Validating everything..."))
+        unit_list, exc_list = self.collect_all_units(provider)
+        early_issue_gen = self.get_early_issues(exc_list)
+        context = UnitValidationContext(provider_list)
+        issue_gen = self.validate_units_in_context(context, unit_list)
+        del context
+        failed = False
+        hidden = 0
+        for issue in itertools.chain(early_issue_gen, issue_gen):
+            show = issue.severity is not Severity.advice
+            if issue.severity is Severity.error:
+                failed = True
+            if ns.strict and issue.severity is Severity.warning:
+                failed = True
+            if ns.deprecated and issue.kind is Problem.deprecated:
+                show = True
+            if ns.strict:
+                failed = True
+                show = True
+            if show:
+                print(issue.relative_to(self.definition.location))
             else:
-                print(str(error))
-        if validation_problem_list or load_problem_list:
+                hidden += 1
+        if hidden > 0:
+            print(ngettext(
+                "NOTE: {0} advice was hidden",
+                "NOTE: {0} advices where hidden",
+                hidden
+            ).format(hidden))
+            print(_(
+                "Run 'manage.py validate -N --strict --deprecated' for details"
+            ))
+        if failed:
+            print(_(
+                "Validation of provider {0} has failed"
+            ).format(provider.name))
             return 1
         else:
             print(_("The provider seems to be valid"))
 
+    def collect_all_units(self, provider):
+        unit_list, exc_list = provider.get_units()
+        return unit_list, exc_list
+
+    def get_early_issues(self, exc_list):
+        # NOTE: exc_list is a list of arbitrary exceptions
+        for exc in exc_list:
+            for exc in exc_list:
+                yield exc2issue(exc)
+        if exc_list:
+            print(_(
+                "NOTE: subsequent units from problematic files are ignored"
+            ))
+
+    def validate_units_in_context(self, context, unit_list):
+        for unit in unit_list:
+            _logger.info(_("Validating unit %s"), unit)
+            for issue in unit.check(context=context, live=True):
+                yield issue
+
     def get_provider(self):
         """
         Get a Provider1 that describes the current provider
@@ -1003,9 +1094,31 @@ class ValidateCommand(ManageCommand):
         This version disables all validation so that we can see totally broken
         providers and let us validate them and handle the errors explicitly.
         """
-        return Provider1.from_definition(
-            # NOTE: don't validate, we want to validate manually
-            self.definition, secure=False, validate=False)
+        # NOTE: don't check or validate, we want to do that manually
+        return Provider1.from_definition(self.definition, secure=False,
+                                         validate=False, check=False)
+
+
+def exc2issue(exc):
+    """
+    Convert an arbitrary exception to an Issue
+
+    :param exc:
+        Any Exception instance
+    :returns:
+        An :class:`plainbox.impl.validation.Issue` instance representing the
+        same error.
+
+    .. note::
+        This should be a single dispatch function so that specialized
+        converters can be used without altering this code all the time.
+    """
+    if isinstance(exc, RFC822SyntaxError):
+        return Issue(
+            exc.msg, Severity.error, Problem.syntax_error,
+            Origin(exc.filename, exc.lineno, exc.lineno))
+    else:
+        return Issue(str(exc), Severity.error, Problem.unknown)
 
 
 @docstring(
