@@ -320,10 +320,10 @@ class ProviderManagerToolTests(TestCase):
             self.tool.main(["validate"])
         self.assertEqual(
             test_io.stdout, (
-                "jobs/broken.txt:1-2: job '2014.com.example::broken', field 'plugin': "
-                "incorrect value supplied\n"
-                "allowed values are: attachment, local, manual, resource"
-                ", shell, user-interact, user-interact-verify, user-verify\n"))
+                "jobs/broken.txt:1-2: job '2014.com.example::broken', field"
+                " 'plugin': allowed values are: attachment, local, manual,"
+                " resource, shell, user-interact, user-interact-verify,"
+                " user-verify\n"))
 
     def test_validate__broken_useless_field(self):
         """
@@ -376,7 +376,7 @@ class ProviderManagerToolTests(TestCase):
             "[Unit Definitions]\n"
             "\tjob 2014.com.example::dummy, from jobs/jobs.txt:1-4\n"
             "[White Lists]\n"
-            "\t'test', from whitelists/test.whitelist:1-1\n"
+            "\t'test', from whitelists/test.whitelist:1\n"
             "[Executables]\n"
             "\t'test.sh'\n"))
 
